@@ -1,17 +1,20 @@
 # -*- coding: utf-8 -*-
 from random import randint
+from typing import Any, Optional, List, TypeVar
 
 import pyodbc
-
 from pip_services3_commons.config import IConfigurable, ConfigParams
 from pip_services3_commons.convert import LongConverter
 from pip_services3_commons.data import PagingParams, DataPage
 from pip_services3_commons.errors import ConnectionException, InvalidStateException
 from pip_services3_commons.refer import IReferenceable, IUnreferenceable, IReferences, DependencyResolver
+from pip_services3_commons.reflect import PropertyReflector
 from pip_services3_commons.run import IOpenable, ICleanable
 from pip_services3_components.log import CompositeLogger
 
-from pip_services3_sqlserver.persistence.SqlServerConnection import SqlServerConnection
+from pip_services3_sqlserver.connect.SqlServerConnection import SqlServerConnection
+
+T = TypeVar('T')  # Declare type variable
 
 
 class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpenable, ICleanable):
@@ -62,7 +65,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
                 return self._model.find_one_and_update(criteria, item, options)
 
         persistence = MySqlServerPersistence()
-        
+
         persistence.configure(ConfigParams.from_tuples(
             "host", "localhost",
             "port", 27017
@@ -88,24 +91,24 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         "options.debug", True
     )
 
-    def __init__(self, table_name=None):
+    def __init__(self, table_name: str = None):
         """
         Creates a new instance of the persistence component.
 
         :param table_name: (optional) a table name.
         """
         # The SQLServer table object.
-        self._table_name = table_name
+        self._table_name: str = table_name
         # The SQLServer database name.
-        self._database_name = None
+        self._database_name: str = None
         # The dependency resolver.
-        self._dependency_resolver = DependencyResolver(self.__default_config)
+        self._dependency_resolver: DependencyResolver = DependencyResolver(self.__default_config)
         # The logger.
-        self._logger = CompositeLogger()
+        self._logger: CompositeLogger = CompositeLogger()
         # The SQLServer connection component.
-        self._connection = None
+        self._connection: SqlServerConnection = None
         # The SQLServer connection pool object.
-        self._client = None
+        self._client: Any = None
 
         self._max_page_size = 100
 
@@ -115,7 +118,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         self.__local_connection = None
         self.__schema_statements = []
 
-    def configure(self, config):
+    def configure(self, config: ConfigParams):
         """
         Configures component by passing configuration parameters.
 
@@ -130,7 +133,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         self._table_name = config.get_as_string_with_default('table', self._table_name)
         self._max_page_size = config.get_as_integer_with_default('options.max_page_size', self._max_page_size)
 
-    def set_references(self, references):
+    def set_references(self, references: IReferences):
         """
         Sets references to dependent components.
 
@@ -155,7 +158,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         """
         self._connection = None
 
-    def __create_connection(self):
+    def __create_connection(self) -> SqlServerConnection:
         connection = SqlServerConnection()
 
         if self.__config:
@@ -166,9 +169,11 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
 
         return connection
 
-    def _ensure_index(self, name, keys, options=None):
+    def _ensure_index(self, name: str, keys: Any, options: Any = None):
         """
         Adds index definition to create it on opening
+        
+        :param name: the index name
         :param keys:  index keys (fields)
         :param options: index options
         """
@@ -196,7 +201,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
 
         self._auto_create_object(builder)
 
-    def _auto_create_object(self, schema_statement):
+    def _auto_create_object(self, schema_statement: str):
         """
         Adds a statement to schema definition.
         This is a deprecated method. Use ensureSchema instead.
@@ -205,7 +210,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         """
         self._ensure_schema(schema_statement)
 
-    def _ensure_schema(self, schema_statement):
+    def _ensure_schema(self, schema_statement: str):
         """
         Adds a statement to schema definition
 
@@ -225,7 +230,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         Todo: override in child classes
         """
 
-    def _convert_to_public(self, value):
+    def _convert_to_public(self, value: Any) -> Any:
         """
         Converts object value from internal to public format.
 
@@ -234,28 +239,34 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         """
         if not value:
             return
-        converted_val = {}
-        for i in range(len(value)):
-            converted_val[value.cursor_description[i][0]] = value[i]
-        return converted_val
 
-    def _convert_from_public(self, value):
+        if isinstance(value, pyodbc.Row):
+            converted_val = {}
+            for i in range(len(value)):
+                converted_val[value.cursor_description[i][0]] = value[i]
+            value = converted_val
+
+        return type('object', (object,), value)
+
+    def _convert_from_public(self, value: Any) -> Any:
         """
         Convert object value from public to internal format.
 
         :param value: an object in public format to convert.
         :return: converted object in internal format.
         """
-        return dict(value)
-
-    def _quote_identifier(self, value):
-        if value is None or value == '':
+        if isinstance(value, dict):
             return value
+        return PropertyReflector.get_properties(value)
+
+    def _quote_identifier(self, value: str) -> str:
+        if value is None or value == '':
+            return ''
         if value[0] == '[':
             return value
         return '[' + value.replace(".", "].[") + ']'
 
-    def is_opened(self):
+    def is_open(self) -> bool:
         """
         Checks if the component is opened.
 
@@ -263,7 +274,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         """
         return self.__opened
 
-    def open(self, correlation_id):
+    def open(self, correlation_id: Optional[str]):
         """
         Opens the component.
 
@@ -280,7 +291,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         if self.__local_connection:
             self._connection.open(correlation_id)
 
-        if not self._connection.is_opened():
+        if not self._connection.is_open():
             self.__opened = False
             raise ConnectionException(correlation_id, "CONNECT_FAILED", "SQLServerF connection is not opened")
         else:
@@ -301,7 +312,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
                 raise ConnectionException(correlation_id, "CONNECT_FAILED",
                                           "Connection to SQLServer failed").with_cause(err)
 
-    def close(self, correlation_id):
+    def close(self, correlation_id: Optional[str]):
         """
         Closes component and frees used resources.
 
@@ -321,7 +332,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         self.__opened = False
         self._client = None
 
-    def clear(self, correlation_id):
+    def clear(self, correlation_id: Optional[str]):
         """
         Clears component state.
 
@@ -339,7 +350,14 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
             raise ConnectionException(correlation_id, "CONNECT_FAILED", "Connection to sqlserver failed").with_cause(
                 err)
 
-    def _request(self, query, params=None):
+    def _request(self, query: str, params: List[str] = None) -> dict:
+        """
+        Request to the database.
+
+        :param query: string with sql query to database
+        :param params: optional list of query parameters
+        :return: result of the query
+        """
         conn = self._client
         cursor = conn.cursor()
         # if params:
@@ -363,15 +381,20 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         cursor.close()
         return rows
 
-    def _create_schema(self, correlation_id):
+    def _create_schema(self, correlation_id: Optional[str]):
+        """
+        TODO add description
+        :param correlation_id:
+        :return:
+        """
         if self.__schema_statements is None or len(self.__schema_statements) == 0:
-            return None
+            return
 
         # Check if table exist to determine weither to auto create objects
         query = "SELECT OBJECT_ID('" + self._table_name + "', 'U') as oid"
         result = self._request(query)
         if result[0] and len(result[0]) > 0 and result[0].oid:
-            return None
+            return
 
         self._logger.debug(correlation_id,
                            'Table ' + self._table_name + ' does not exist. Creating database objects...')
@@ -383,7 +406,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         except Exception as err:
             self._logger.error(correlation_id, err, 'Failed to autocreate database object')
 
-    def _generate_columns(self, values):
+    def _generate_columns(self, values: Any) -> str:
         """
         Generates a list of column names to use in SQL statements like: "column1,column2,column3"
 
@@ -400,7 +423,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
 
         return result
 
-    def _generate_parameters(self, values):
+    def _generate_parameters(self, values: Any) -> str:
         """
         Generates a list of value parameters to use in SQL statements like: "?,?,?"
 
@@ -417,7 +440,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
 
         return result
 
-    def _generate_set_parameters(self, values):
+    def _generate_set_parameters(self, values: Any) -> str:
         """
         Generates a list of column sets to use in UPDATE statements like: column1=?1,column2=?2
 
@@ -434,7 +457,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
 
         return result
 
-    def _generate_values(self, values):
+    def _generate_values(self, values: Any) -> List[Any]:
         """
         Generates a list of column parameters
 
@@ -443,7 +466,8 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         """
         return list(values.values())
 
-    def get_page_by_filter(self, correlation_id, filter, paging, sort, select):
+    def get_page_by_filter(self, correlation_id: Optional[str], filter: Any, paging: PagingParams,
+                           sort: Any, select: Any) -> DataPage:
         """
         Gets a page of data items retrieved by a given filter and sorted according to sort parameters.
 
@@ -496,7 +520,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         else:
             return DataPage(items)
 
-    def get_count_by_filter(self, correlation_id, filter):
+    def get_count_by_filter(self, correlation_id: Optional[str], filter: Any) -> int:
         """
         Gets a number of data items retrieved by a given filter.
         This method shall be called by a public getCountByFilter method from child class that
@@ -520,7 +544,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
 
         return count
 
-    def get_list_by_filter(self, correlation_id, filter, sort, select):
+    def get_list_by_filter(self, correlation_id: Optional[str], filter: Any, sort: Any, select: Any) -> List[T]:
         """
         Gets a list of data items retrieved by a given filter and sorted according to sort parameters.
         This method shall be called by a public getListByFilter method from child class that
@@ -541,9 +565,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         if sort and len(sort) > 0:
             query += " ORDER BY " + sort
 
-        params = self._create_params()
-
-        result = self._request(query, params)
+        result = self._request(query)
         items = result['recordset']
 
         if items is not None:
@@ -553,7 +575,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
 
         return items
 
-    def get_one_random(self, correlation_id, filter):
+    def get_one_random(self, correlation_id: Optional[str], filter: Any) -> T:
         """
         Gets a random item from items that match to a given filter.
         This method shall be called by a public getOneRandom method from child class that
@@ -567,8 +589,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         if filter and filter != '':
             query += " WHERE " + filter
 
-        params = self._create_params()
-        result = self._request(query, params)
+        result = self._request(query)
 
         query = "SELECT * FROM " + self._quote_identifier(self._table_name)
 
@@ -592,7 +613,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         item = self._convert_to_public(item)
         return item
 
-    def create(self, correlation_id, item):
+    def create(self, correlation_id: Optional[str], item: T) -> Optional[T]:
         """
         Creates a data item.
 
@@ -620,7 +641,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
 
         return new_item
 
-    def delete_by_filter(self, correlation_id, filter):
+    def delete_by_filter(self, correlation_id: Optional[str], filter: Any):
         """
         Deletes data items that match to a given filter.
 
@@ -632,8 +653,7 @@ class SqlServerPersistence(IReferenceable, IUnreferenceable, IConfigurable, IOpe
         if filter and filter != '':
             query += " WHERE " + filter
 
-        params = self._create_params()
-        result = self._request(query, params)
+        result = self._request(query)
 
         count = result['rowsAffected'][0] if result and result['rowsAffected'] else 0
 
